use lib 't/tests';
use Test::Webservice::InterMine::PathFeature;
use Test::Webservice::InterMine::Join;
use Test::Webservice::InterMine::SortOrder;
use Test::Webservice::InterMine::PathDescription;
Test::Class->runtests;
